@@ -33,6 +33,16 @@ _CDICT_FN = 'cdict.pickle'
 class BaseTagger(BaseParser):
     """
     A base class for the project's taggers.
+
+    Args:
+
+    **embs**: `dict` with paths to the embeddings file as keys and
+    corresponding embeddings models as values. If tagger needs to load any
+    embeddings model, firstly, model is looked up it in that `dict`.
+
+    During init, **embs** is copied to the `emb` attribute of the creating
+    object, and this attribute may be used further to share already loaded
+    embeddings with another taggers.
     """
 
     def __err_hideattr(self, name):
@@ -55,10 +65,15 @@ class BaseTagger(BaseParser):
     _train_done = \
         property(lambda self: self.__err_hideattr(self, '_train_done'))
 
-    def __init__(self):
+    @property
+    def embs(self):
+        return self._embs
+
+    def __init__(self, embs=None):
         super().__init__()
         self._model = None
         self._ds = None
+        self._embs = {} if embs is None else embs.copy()
 
     def load_train_corpus(self, corpus, append=False, test=None, seed=None):
         """Loads the train corpus.
@@ -121,7 +136,7 @@ class BaseTagger(BaseParser):
                 sentences, emb_type=word_emb_type, emb_path=word_emb_path,
                 emb_model_device=word_emb_model_device,
                 transform_kwargs=word_transform_kwargs,
-                next_emb_params=word_next_emb_params,
+                next_emb_params=word_next_emb_params, embs=self.embs,
                 loglevel=0 if log_file is None else
                          1 if log_file == sys.stdin else
                          2
@@ -195,8 +210,10 @@ class BaseTagger(BaseParser):
                     loglevel=int(log_file is not None)
                 )
                 if not res_:
-                    res_ = ds_.transform_collate(sentences,
-                                                 batch_size=batch_size)
+                    res_ = ds_.transform_collate(
+                        sentences, batch_size=batch_size,
+                        collate_kwargs={'with_lens': False}
+                    )
                 res.append(res_)
             elif typ == 't':
                 res.append(ds_.transform_collate(
@@ -242,7 +259,7 @@ class BaseTagger(BaseParser):
             config = json.loads(f.read())
             for name, cfg in config.items():
                 WordEmbeddings.apply_config(ds.get_dataset(name), cfg,
-                                            device=device)
+                                            device=device, embs=self.embs)
         if log_file:
             print('done.', file=log_file)
         if for_self:
@@ -269,7 +286,8 @@ class BaseTagger(BaseParser):
         They are editable, but you are allowed to change only the device name.
         Any other changes most likely won't allow the tagger to load.
         """
-        assert self._ds, "ERROR: The tagger doesn't have a dataset to save"
+        assert self._ds is not None, \
+               "ERROR: The tagger doesn't have a dataset to save"
         assert self._model, "ERROR: The tagger doesn't have a model to save"
         self._save_dataset(name)
         model_config_fn, model_fn, _, _, cdict_fn = \
