@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
-# MorDL project: NE tagger
+# MorDL project: DEPREL tagger
 #
 # Copyright (C) 2020-present by Sergei Ternovykh, Anastasiya Nikiforova
 # License: BSD, see LICENSE for details
 """
-Provides a Named-entity tagger class.
+Provides classes of DEPREL taggers.
 """
 from copy import deepcopy
 import itertools
@@ -16,16 +16,133 @@ from mordl.deprel_tagger_model import DeprelTaggerModel
 import torch
 from typing import Iterator
 
-WINDOW_LEFT, WINDOW_RIGHT = 2, 2
-PAD = '[PAD]'
-PAD_TOKEN = {'ID': '0', 'FORM': PAD, 'LEMMA': PAD,
-                        'UPOS': PAD, 'FEATS': {}, 'DEPREL': None}
-
 
+# LAS on *SynTagRus*: 97.60
+# with supp_tagger of `DeprelSeqTagger` type: 97.60
 class DeprelTagger(FeatTagger):
     """
-    Named-entity tagger class. We use the feature 'NE' of MISC field as the
-    place where Named-entity tags are stored.
+    A DEPREL tagger class.
+
+    Args:
+
+    **feats_prune_coef** (`int`): feature prunning coefficient which allows to
+    eliminate all features that have a low frequency. For each UPOS tag, we
+    get a number of occurences of the most frequent feature from FEATS field,
+    divide it by **feats_prune_coef** and use only those features, number of
+    occurences of which is greater than that value, to improve the prediction
+    quality.
+    * `feats_prune_coef=0` means "do not use feats";
+    * `feats_prune_coef=None` means "use all feats";
+    * default `feats_prune_coef=6`.
+
+    **supp_tagger**: an object of another DEPREL tagger which method
+    `.predict()` has the same signature as `DeprelTagger.predict()` and no
+    excess `'root'` tags in the return. Object of `DeprelSeqTagger` may be
+    used here.
+
+    **embs**: `dict` with paths to the embeddings file as keys and
+    corresponding embeddings models as values. If tagger needs to load any
+    embeddings model, firstly, model is looked up it in that `dict`.
+
+    During init, **embs** is copied to the `emb` attribute of the creating
+    object, and this attribute may be used further to share already loaded
+    embeddings with another taggers.
+    """
+    def __init__(self, feats_prune_coef=6, supp_tagger=None, embs=None):
+        super().__init__('DEPREL', feats_prune_coef=feats_prune_coef,
+                         embs=embs)
+        self._supp_tagger = supp_tagger
+
+    def predict(self, corpus, with_orig=False, batch_size=BATCH_SIZE,
+                split=None, clone_ds=False, save_to=None, log_file=LOG_FILE):
+        """Predicts tags in the DEPREL field of the corpus.
+
+        Args:
+
+        **corpus**: a corpus which will be used for feature extraction and
+        predictions. May be either a name of the file in *CoNLL-U* format or a
+        list/iterator of sentences in *Parsed CoNLL-U*.
+
+        **with_orig** (`bool`): if `True`, instead of only a sequence with
+        predicted labels, returns a sequence of tuples where the first element
+        is a sentence with predicted labels and the second element is the
+        original sentence. `with_orig` can be `True` only if `save_to` is
+        `None`. Default `with_orig=False`.
+
+        **batch_size** (`int`): number of sentences per batch. Default
+        `batch_size=64`.
+
+        **split** (`int`): number of lines in each split. Allows to process a
+        large dataset in pieces ("splits"). Default `split=None`, i.e. process
+        full dataset without splits.
+
+        **clone_ds** (`bool`): if `True`, the dataset is cloned and
+        transformed. If `False`, `transform_collate` is used without cloning
+        the dataset. There is no big differences between the variants. Both
+        should produce identical results.
+
+        **save_to**: file name where the predictions will be saved.
+
+        **log_file**: a stream for info messages. Default is `sys.stdout`.
+
+        Returns corpus with tags predicted in the DEPREL field.
+        """
+        assert self._ds is not None, \
+               "ERROR: The tagger doesn't have a dataset. Call the train() " \
+               'method first'
+        assert not with_orig or save_to is None, \
+               'ERROR: `with_orig` can be True only if save_to is None'
+        args, kwargs = get_func_params(FeatTagger.predict, locals())
+
+        kwargs['save_to'] = None
+
+        corpus2 = None
+        if self._supp_tagger:
+            kwargs2 = kwargs.copy()
+            kwargs2['with_orig'] = True
+            corpus2 = self._supp_tagger.predict(*args, **kwargs2)
+
+        corpus = super().predict(*args, **kwargs)
+
+        def add_root(corpus):
+            for sent in corpus:
+                sent0 = sent[0] if isinstance(sent, tuple) \
+                               and not isinstance(sent[0], tuple) \
+                               and not isinstance(sent[1], tuple) else \
+                        sent
+                if isinstance(sent0, tuple):
+                    sent0 = sent0[0]
+                if corpus2:
+                    sent2 = next(corpus2)[1]
+                    if isinstance(sent2, tuple):
+                        sent2 = sent2[0]
+                for i, tok in enumerate(sent0):
+                    if tok['HEAD'] == '0':
+                        tok['DEPREL'] = 'root'
+                    elif corpus2 and tok['DEPREL'] == 'root':
+                        tok['DEPREL'] = sent2[i]['DEPREL']
+                yield sent
+
+        corpus = add_root(corpus)
+
+        if save_to:
+            self.save_conllu(corpus, save_to, log_file=None)
+            corpus = self._get_corpus(save_to, asis=True, log_file=log_file)
+
+        return corpus
+
+NODES_UP, NODES_DOWN = 2, 2
+PAD = '[PAD]'
+PAD_TOKEN = {'ID': '0', 'FORM': PAD, 'LEMMA': PAD,
+                         'UPOS': PAD, 'FEATS': {}, 'DEPREL': None}
+
+class DeprelSeqTagger(FeatTagger):
+    """
+    A DEPREL tagger class with sequence model.
+
+    **WARNING**: Works long, takes huge amoont of RAM, but accuracy is 
+    less than of `DeprelTagger`. But may be useful as suppplement tagger
+    for it.
 
     Args:
 
@@ -46,43 +163,43 @@ class DeprelTagger(FeatTagger):
     @staticmethod
     def _preprocess_corpus(corpus):
 
-        def window_right(sent, id_, ids, chains, window_len):
-            link_ids = chains.get(id_, [])
+        def nodes_down(sent, id_, ids, nodes, span_len):
+            link_ids = nodes.get(id_, [])
             res = []
             for link_id in link_ids:
                 idx = ids[link_id]
                 token = sent[idx]
-                if window_len == 1:
+                if span_len == 1:
                     res.append([token])
                 else:
-                    for window_right_ in window_right(sent, link_id, ids,
-                                                      chains, window_len - 1):
-                        res.append([token] + window_right_)
+                    for nodes_down_ in nodes_down(sent, link_id, ids,
+                                                  nodes, span_len - 1):
+                        res.append([token] + nodes_down_)
             if not res:
-                res = [[PAD_TOKEN] * window_len]
+                res = [[PAD_TOKEN] * span_len]
             return res
 
-        def next_sent(sent, upper_sent, id_, ids, chains):
-            link_ids = chains.get(id_, [])
+        def next_sent(sent, upper_sent, id_, ids, nodes):
+            link_ids = nodes.get(id_, [])
             for link_id in link_ids:
                 idx = ids[link_id]
                 token = sent[idx]
                 label = token['DEPREL']
-                s = upper_sent[-WINDOW_LEFT:]
+                s = upper_sent[-NODES_UP:]
                 s.append(token)
-                for window_right_ in window_right(sent, link_id, ids,
-                                                  chains, WINDOW_RIGHT):
-                    s_ = ([PAD_TOKEN] * (WINDOW_LEFT + 1 - len(s))) \
-                       + s[:] + window_right_
+                for nodes_down_ in nodes_down(sent, link_id, ids,
+                                              nodes, NODES_DOWN):
+                    s_ = ([PAD_TOKEN] * (NODES_UP + 1 - len(s))) \
+                       + s[:] + nodes_down_
                     yield s_, idx, label
-                for data_ in next_sent(sent, s, link_id, ids, chains):
+                for data_ in next_sent(sent, s, link_id, ids, nodes):
                     yield data_
 
         res_corpus, labels, restore_data = [], [], []
         for i, sent in enumerate(corpus):
             if isinstance(sent, tuple):
                 sent = sent[0]
-            root_id, root_token, ids, chains = None, None, {}, {}
+            root_id, root_token, ids, nodes = None, None, {}, {}
             for idx, token in enumerate(sent):
                 id_, head = token['ID'], token['HEAD']
                 if head:
@@ -91,10 +208,10 @@ class DeprelTagger(FeatTagger):
                     if head == '0':
                         root_id, root_token = id_, token
                     ids[id_] = idx
-                    chains.setdefault(head, []).append(id_)
+                    nodes.setdefault(head, []).append(id_)
             if root_id:
                 for s, idx, label in next_sent(sent, [root_token],
-                                               root_id, ids, chains):
+                                               root_id, ids, nodes):
                     res_corpus.append(s)
                     labels.append(label)
                     restore_data.append((i, idx))
@@ -125,7 +242,7 @@ class DeprelTagger(FeatTagger):
         res = super()._prepare_corpus(corpus, fields,
                                       tags_to_remove=tags_to_remove)
         res = list(res)
-        res[-1] = [x[WINDOW_LEFT] for x in res[-1]]
+        res[-1] = [x[NODES_UP] for x in res[-1]]
         return tuple(res)
 
     def load(self, name, device=None, dataset_device=None, log_file=LOG_FILE):
@@ -149,8 +266,7 @@ class DeprelTagger(FeatTagger):
 
     def predict(self, corpus, with_orig=False, batch_size=BATCH_SIZE,
                 split=None, clone_ds=False, save_to=None, log_file=LOG_FILE):
-        """Predicts tags in the specified feature of the FEATS field of the
-        corpus.
+        """Predicts tags in the DEPREL field of the corpus.
 
         Args:
 
@@ -180,8 +296,7 @@ class DeprelTagger(FeatTagger):
 
         **log_file**: a stream for info messages. Default is `sys.stdout`.
 
-        Returns corpus with predicted values of certain feature in the FEATS
-        field.
+        Returns corpus with tags predicted in the DEPREL field.
         """
         assert self._ds is not None, \
                "ERROR: The tagger doesn't have a dataset. Call the train() " \
@@ -295,7 +410,7 @@ class DeprelTagger(FeatTagger):
               upos_emb_dim=300, emb_out_dim=512, lstm_hidden_dim=256,
               lstm_layers=2, lstm_do=0, bn1=True, do1=.2, bn2=True, do2=.5,
               bn3=True, do3=.4, seed=None, log_file=LOG_FILE):
-        """Creates and trains a feature tagger model.
+        """Creates and trains the DEPREL tagger model.
 
         During training, the best model is saved after each successful epoch.
 
