@@ -38,10 +38,18 @@ class FeatTagger(BaseTagger):
     * default `feats_prune_coef=6`.
 
     **NB**: the argument is relevant only if **feat** is not from FEATS field.
+
+    **embs**: `dict` with paths to the embeddings file as keys and
+    corresponding embeddings models as values. If tagger needs to load any
+    embeddings model, firstly, model is looked up it in that `dict`.
+
+    During init, **embs** is copied to the `emb` attribute of the creating
+    object, and this attribute may be used further to share already loaded
+    embeddings with another taggers.
     """
-    def __init__(self, feat, feats_prune_coef=6):
+    def __init__(self, feat, feats_prune_coef=6, embs=None):
         assert feat != 'UPOS', 'ERROR: for UPOS field use UposTagger class'
-        super().__init__()
+        super().__init__(embs=embs)
         if feat[0] == ':':
             feat = 'FEATS' + feat
         self._field = feat
@@ -357,8 +365,11 @@ class FeatTagger(BaseTagger):
         if self._feats_prune_coef != 0:
             [x.update({'LEMMA': x['FORM']})
                  for x in self._train_corpus for x in x]
-            self._cdict = CorpusDict(corpus=self._train_corpus,
-                                     format='conllu_parsed', log_file=log_file)
+            self._cdict = CorpusDict(
+                corpus=self._train_corpus
+                     + (self._test_corpus if self._test_corpus else []),
+                format='conllu_parsed', log_file=log_file
+            )
             self._save_cdict(save_as + '.cdict.pickle')
             if log_file:
                 print(file=log_file)
