@@ -38,8 +38,8 @@ class LemmaTagger(BaseTagger):
     * `feats_prune_coef=None` means "use all feats";
     * default `feats_prune_coef=6`.
     """
-    def __init__(self, field='LEMMA', feats_prune_coef=6):
-        super().__init__()
+    def __init__(self, field='LEMMA', feats_prune_coef=6, embs=None):
+        super().__init__(embs=embs)
         self._field = field
         self._feats_prune_coef = feats_prune_coef
 
@@ -252,8 +252,8 @@ class LemmaTagger(BaseTagger):
         kwargs['save_to'] = None
 
         cdict = self._cdict
-        yof = len([x for x in cdict._wforms if 'ё' in x])
-        yol = len([x for x in cdict._lemmata if 'ё' in x])
+        yof = len([x for x in cdict._wforms if 'ё' in x or 'Ё' in x])
+        yol = len([x for x in cdict._lemmata if 'ё' in x or 'Ё' in x])
         remove_yo = yol / yof < 10
 
         def apply_editops(str_from, upos, ops_t, isfirst):
@@ -272,9 +272,10 @@ class LemmaTagger(BaseTagger):
                         ))
                         if str_from_:
                             str_from = str_from_
-                            if 'ё' in str_from:
+                            if 'ё' in str_from or 'Ё' in str_from:
                                 if not cdict.lemma_isknown(str_from, upos):
-                                    str_from_ = str_from.replace('ё', 'е')
+                                    str_from_ = str_from.replace('ё', 'е') \
+                                                        .replace('Ё', 'Е')
                                     if remove_yo \
                                     or cdict.lemma_isknown(str_from_, upos):
                                         str_from = str_from_
@@ -484,8 +485,11 @@ class LemmaTagger(BaseTagger):
 
         args, kwargs = get_func_params(LemmaTagger.train, locals())
 
-        self._cdict = CorpusDict(corpus=self._train_corpus,
-                                 format='conllu_parsed', log_file=log_file)
+        self._cdict = CorpusDict(
+            corpus=self._train_corpus
+                 + (self._test_corpus if self._test_corpus else []),
+            format='conllu_parsed', log_file=log_file
+        )
 
         if log_file:
             print('\nPreliminary trainset preparation:', file=log_file)
